const SustainableWebDesign = require("./sustainable-web-design");

describe("sustainable web design model", () => {
  const swd = new SustainableWebDesign();
  const averageWebsiteInBytes = 2257715.2;

  describe("energyPerByteByComponent", () => {
    it("should return a object with numbers for each system component", () => {
      const groupedEnergy = swd.energyPerByteByComponent(averageWebsiteInBytes);

      expect(groupedEnergy.consumerDeviceEnergy).toBeCloseTo(0.00088564, 8);
      expect(groupedEnergy.networkEnergy).toBeCloseTo(0.00023844, 8);
      expect(groupedEnergy.productionEnergy).toBeCloseTo(0.0003236, 8);
      expect(groupedEnergy.dataCenterEnergy).toBeCloseTo(0.00025547, 8);
    });
  });

  describe("energyPerByte", () => {
    it("should return a number in kilowatt hours for the given data transfer in bytes", () => {
      const energyForTransfer = swd.energyPerByte(averageWebsiteInBytes);
      expect(energyForTransfer).toBeCloseTo(0.00170316, 7);
    });
  });

  describe("perByte", () => {
    it("should return a single number for CO2 emissions", () => {
      expect(typeof swd.perByte(2257715.2)).toBe("number");
    });
  });

  describe("energyPerVisit", function () {
    it("should return a number", () => {
      expect(typeof swd.energyPerVisit(averageWebsiteInBytes)).toBe("number");
    });

    it("should calculate the correct energy", () => {
      expect(swd.energyPerVisit(averageWebsiteInBytes)).toBe(
        0.0004513362121582032
      );
    });
  });

  describe("emissionsPerVisitInGrams", function () {
    it("should calculate the correct co2 per visit", () => {
      const averageWebsiteInBytes = 2257715.2;
      const energy = swd.energyPerVisit(averageWebsiteInBytes);
      expect(swd.emissionsPerVisitInGrams(energy)).toEqual(0.2);
    });

    it("should accept a dynamic KwH value", () => {
      const averageWebsiteInBytes = 2257715.2;
      const energy = swd.energyPerVisit(averageWebsiteInBytes);
      expect(swd.emissionsPerVisitInGrams(energy, 245)).toEqual(0.11);
    });
  });

  describe("annualEnergyInKwh", function () {
    it("should calculate the correct energy in kWh", () => {
      expect(swd.annualEnergyInKwh(averageWebsiteInBytes)).toBe(27092582400);
    });
  });

  describe("annualEmissionsInGrams", function () {
    it("should calculate the corrent energy in grams", () => {
      expect(swd.annualEmissionsInGrams(averageWebsiteInBytes)).toBe(
        27092582400
      );
    });
  });

  describe("annualSegmentEnergy", function () {
    it("should return the correct values", () => {
      expect(swd.annualSegmentEnergy(averageWebsiteInBytes)).toEqual({
        consumerDeviceEnergy: 1174011.9,
        dataCenterEnergy: 338657.28,
        networkEnergy: 316080.13,
        productionEnergy: 428965.89,
      });
    });
  });
});
